<?php
/*
	"If in the course of a lecture I stated an unsolved problem, the 
	chances were [John von Neumann would] come to me at the end of the 
	lecture with the complete solution scribbled on a slip of paper."
	  -- Miodrag Petkovic, "Famous puzzles of great mathematicians"
*/

/* HTML.PHP - create the $html array for the web templates 

The $html array is created from two files in the config('htmdir') directory, 
HTML.INI which defines the HTML that dynamically create the menus as well as 
some very basic strings and error messgaes, and TEMPLATES.PHP wchich defines 
all the web templates.

Using the function adds almost nothing in overhead and it eliminates the 
creation and passing around a global class. The function html() can also 
return the entire $html array:

	$html = html();
	echo $html['id'];

In the web templates the $html array is always in scope:

	<title>{$html['title']}</title>

See file HTML, see file TEMPLATES, see file THEMES.

*/


/* the initialize functiton */

function _config_html() {
	html();
}

/* html - interface to the $html array */

// initializes on first use and indexes into the $html array
// see also config() in MOD/CONFIG.PHP

function html($var = NULL, $val = NULL) {
static $html = array();

	if ($html == array()) {

// the default theme is always loaded; an additional theme, if set, is then
// loaded to overwrite existing data and append new data

		$hd = config('htmdir');
		$td = config('themedir');
		$html = configfile($hd.'html.ini',NULL,TRUE);
		$html['htmdir'] = $hd;
		$html['themedir'] = $hd.$td;
		debug("include {$hd}templates.php");
		include $hd.'templates.php';
		if ($td) {
			$html = configfile($hd.$td.'html.ini',$html,TRUE);
			debug("include $hd{$td}templates.php");
			include $hd.$td.'templates.php';
		}

		$html['host'] = 'http://'.$_SERVER['HTTP_HOST'];
		$html['done'] = $html['host'].$_SERVER['REQUEST_URI'];
		$html['host'] .= $_SERVER['PHP_SELF'];
		$html['base'] = basename($_SERVER['PHP_SELF']);
		$html['title'] = config('title');
		$html['version'] = $GLOBALS['VERSION'];

		$arg = config('arg');
		if (isset($html[$arg]))				// [$arg] in
			foreach ($html[$arg] as $k => $v)	//  html.ini
				$html[$k] = $v;

		$html['name'] = config('name');
		$html['description'] = config('description');
		$html['sitetitle'] = config('sitetile');

		htmlmenu($html);
		htmllogin($html);
		htmlnavmenu($html);
		htmlmakelinks($html);
		htmlimport($html);

# adjustment based on context; the data should some how do this...
		$op = getvar('op');
		if (config('hidenavmenu') && $op && $op != 'read')
			$html['navmenu'] = '';
		if (!config('allowcomments'))
			$html['gotocomments'] = '';
	}

	if ($var === NULL)
		return $html;

	if ($val !== NULL) {
		if (isset($html[$var]) && is_array($html[$var]))
			return $html[$var][] = $val;
		return $html[$var] = $val;
	}

	if (isset($html[$var]))
		return $html[$var];

	return '';
}


/* htmltempl - get template */

// complicated only for error checking

function htmltempl($name, $file = 0) {

	if (!$file)
		$templ = html($name);
	else {
		$templ = '';
		if (is_file($name))
			$templ = file_get_contents($name);
	}

	if (!$templ) {
		debug("'$name' not found");
		if (config('shownotfounds') && !config('nodisplaymessages'))
			$templ = geterror('TEMPLATE');
	}

	return $templ;
}

/* htmldirect - check if template name is in 'direct' for direct printing */
 
function htmldirect($name) {

	$templs = html('direct');
	if ($templs)
		return in_array($name,$templs);
	return FALSE;
}


/* htmlseterror - set template accessible error message */

function htmlseterror($name, $id = 'comment_message') {

	$error = htmlgeterror($name);
	return html($id,$error);
}


/* htmlgeterror - error message based on name or last html() error */

function htmlgeterror($name = NULL, $id = 'comment_message') {

	if ($name == NULL)
		return html($id);		// default "global" message

	$error = html($name);
	if (!$error)
		return "error '$name' error!";	//  oh no, an error error!

	return $error;
}


/* geterror - just get the error message or not */

function geterror($name) {

	return html($name);
}


/* make_url - a single place for all generated URLs */

// this is a complicated function, however, it removes having to maintain 
// explicit URL strings in the code and templates (see htmlmakelinks()), 
// and, when we change the URL format, that will only have to be done here

// as with all "complicated" code here, this will eventually be reshaped 
// into something more understandable and more efficient...

// this code also violates our rule of "Keep inline strings in the code 
// to a minimum." in this case they are 'back', 'help' and 'start'

// additionally, at some point all this "backwards compatibility" stuff 
// will go away and only "reduced URLs" will be supported -- when that 
// happens all these comments of course will go away too!

function make_url($op = '', $id = '') {

	$host = config('siteurl');

	if ($id === '')
		$id = getvar('id');

	$arg = getvar('arg');
	$url = ($arg) ? '?'.$arg : '?';

	if ($op === '')
		return $host.$url;

// we do not like special cases, but not sure how to do this one otherwise...
	if ($op == 'back') {
		if (($start = getvar('start')) && $start > 0)
			$url .= '&amp;start='.$start;
		return $host.$url;
	}
// this is a "backwards" compatibility change
	if ($op == 'help')
		return $host.'?help#'.$id;

	if ($arg)
		$url .= '&amp;';

	$url .= $op;

	if ($id)
		$url .= '='.$id;

	if (($start = getvar('start')) && $start > 0)
		$url .= '&amp;start='.$start;

	return $host.$url;
}


/*
Some HTML is dynamic, depending on the "operation" these form that HTML.
(There are getting to be too many $html member names and they are all 
different with no consistency... this may become problematic.)
*/

// the main menu is dynamically created by the number of sections (see 
// SECTIONS.INI which is parsed by CONFIG.PHP) and the result is placed in 
// $html['menu'] (which a theme can pre-define if it wants) and it consists 
// of a $html['menustr'] and a $html['menuafter'] -- both defined in HTML.INI
// -- for each section
# this can be hard to understand and it can be annoying...

function htmlmenu(&$html) {

	if (isset($html['menu']))		// already have one?
		return;
	if (!$html['menustr'])			// no makings?
		return;

	$op = getvar('op');			// what we are doing
	$arg = getvar('arg');			// where we are doing it

	if ($op == 'page') {			// no menu at all for pages
		$html['menu'] = '';
		return;
	}

	if (isset($html[$op])) {
		$h = $html[$op];		// possibly a sub-menu?
	}
	else {					// no, so make the menu
		$s = $html['menustr'];
		$a = $html['menuafter'];
		$sections = config('sections');
		foreach ($sections as $name => $sec) // skip "hidden"
			if (isset($sec['hidden']))
				unset($sections[$name]);
		$h = html_elements($sections,$s,$arg,$a);
	}

	$html['menu'] = str_replace('$1',$h,$html['menublock']);
}


// okay, for this one we messed up a little and hopefully it will be 
// fixed soon... the navigation menu (or sidebar) is $html['navmenu'] and is
// defined in TEMPLATES.PHP and it has a dynamically created component, 
// $html['navstra'] (nav string links) which is comprised of a $html['navstr'] 
// for each section (not too confusing, is it?)

function htmlnavmenu(&$html) {

	if (!$html['navstr'])
		return;
	if (isset($html['navstra']))
		return;

	$arg = getvar('arg');

	$sections = config('sections');
	foreach ($sections as $name => $sec)
		if (isset($sec['hidden']))
			unset($sections[$name]);

	$s = $html['navstr'];
	$html['navstra'] = html_elements($sections,$s,$arg);
}


/* htmllogin - make the login/logout HTML available or not */

function htmllogin(&$html) {

	if (!config('visitorcode'))
		return;

	if (getvar('op') != 'read')
		return;

	$html['redirect'] = htmlspecialchars($_SERVER['QUERY_STRING']);
	if ($html['redirect'])
		$html['redirect'] = '?'.$html['redirect'];

	$from = visitor_check();		// is someone logged in?

	if ($from !== '') {
		$html['logoutstr'] = str_replace('$1',$from,$html['logoutstr']);
		$html['loginout'] = $html['logoutform'];
	}
	else {
		$html['loginout'] = $html['loginform'];
	}
}


/* htmlmakelinks - a single place for all created URLs used by templates */

// see [makeurl] in HTM/HTML.INI
// use the data command in Admin and view the $html array to see what these are

function htmlmakelinks(&$html) {

	if (!isset($html['makeurl']))
		return;

	$urls = $html['makeurl'];
	foreach ($urls as $name => $args) {
		$a = explode(',',$args);
		if (!isset($a[1])) $a[1] = '';
		$html[$name] = make_url($a[0],$a[1]);
	}

	$html['arg'] = getvar('arg');
	$html['op'] = getvar('op');
	$html['id'] = getvar('id');

// well, a hardcoded, special case! not sure how to do it otherwise...
	if ($html['id'])
		$html['back'] .= '#'.$html['id'];
}

// experimental

function htmlimport(&$html) {

	$html['imports'] = '';
	if (isset($html['import'])) {
		if (!is_array($html['import']))
			$html['imports'] = "\n@import url({$html['import']});";
		else
		foreach ($html['import'] as $i)
			$html['imports'] .= "\n@import url($i);";
		$html['imports'] .= "\n";
	}
}


/*
This is the one "produce HTML based on array data" function. It is used to 
eliminate having code in the templates (and therefore the need for anything 
like a "template engine").
*/

/* html_elements - form HTML based on array data */

function html_elements($data, $html, $sel = '', $after = '') {

	$htm = '';

	if (!is_array(current($data))) {			// single array
		foreach ($data as $key => $value) {
			$str = str_replace('$1',$key,$html); 	// note '$html'
			$str = str_replace('$2',$value,$str);	// note '$str'
			$htm .= $str."\n";
		}

		return $htm;
	}

	$i = 0;
	$c = count($data);
	foreach ($data as $key => $value) {
		$names = array_keys($value);
		if (++$i == $c) $after = '';

		$str = str_replace('$key',$key,$html);

		$selected = ($key == $sel) ? 'selected' : '';
		$str = str_replace('$selected',$selected,$str);

		foreach ($names as $n) {
			$id = '$'.$n;
			$str = str_replace($id,$value[$n],$str);
		}

		$htm .= $str.$after;
	}

	return $htm;
}
